"""API Keys resource."""

from typing import Dict, Any, Literal
from ..http_client import HttpClient


class ApiKeys:
    """API Keys resource."""

    def __init__(self, http: HttpClient):
        self.http = http

    def list(self) -> list:
        """List all API keys.
        
        Returns:
            List of API key dictionaries
        """
        return self.http.get("/apikeys")

    def create(
        self,
        name: str,
        environment: Literal["sandbox", "production"],
    ) -> Dict[str, Any]:
        """Create a new API key.
        
        Args:
            name: Name for the API key
            environment: Environment (sandbox or production)
            
        Returns:
            Created API key dictionary
        """
        data = {
            "name": name,
            "environment": environment,
        }
        return self.http.post("/apikeys", data=data)

    def delete(self, key_id: str) -> None:
        """Delete an API key.
        
        Args:
            key_id: The API key ID to delete
        """
        self.http.delete(f"/apikeys/{key_id}")

    def toggle_status(self, key_id: str) -> Dict[str, Any]:
        """Toggle API key status (active/inactive).
        
        Args:
            key_id: The API key ID to toggle
            
        Returns:
            Updated API key dictionary
        """
        return self.http.put(f"/apikeys/{key_id}/toggle")

    def regenerate_secret(self, key_id: str) -> Dict[str, Any]:
        """Regenerate secret key.
        
        Args:
            key_id: The API key ID to regenerate secret for
            
        Returns:
            Updated API key dictionary with new secret
        """
        return self.http.post(f"/apikeys/{key_id}/regenerate-secret")




"""Webhooks resource."""

import hmac
import hashlib
import json
from typing import Dict, Any, Optional
from ..http_client import HttpClient


class Webhooks:
    """Webhooks resource."""

    def __init__(self, http: HttpClient):
        self.http = http

    def get_settings(self) -> Dict[str, Any]:
        """Get webhook settings.
        
        Returns:
            Webhook settings dictionary
        """
        return self.http.get("/webhooks/settings")

    def update_settings(
        self,
        webhook_url: Optional[str] = None,
        enabled: Optional[bool] = None,
    ) -> Dict[str, Any]:
        """Update webhook settings.
        
        Args:
            webhook_url: New webhook URL
            enabled: Enable or disable webhooks
            
        Returns:
            Updated webhook settings dictionary
        """
        data = {}
        if webhook_url is not None:
            data["webhookUrl"] = webhook_url
        if enabled is not None:
            data["enabled"] = enabled

        return self.http.post("/webhooks/settings", data=data)

    def get_logs(self) -> list:
        """Get webhook logs.
        
        Returns:
            List of webhook log dictionaries
        """
        return self.http.get("/webhooks/logs")

    def test(self) -> Dict[str, Any]:
        """Test webhook delivery.
        
        Returns:
            Test webhook response
        """
        return self.http.post("/webhooks/test")

    def regenerate_secret(self) -> Dict[str, Any]:
        """Regenerate webhook secret.
        
        Returns:
            Dictionary containing new webhook secret
        """
        return self.http.post("/webhooks/regenerate-secret")

    @staticmethod
    def verify(payload: Any, signature: str, secret: str) -> bool:
        """Verify webhook signature.
        
        Args:
            payload: The webhook payload (dict or string)
            signature: The signature from x-lipana-signature header
            secret: Your webhook secret
            
        Returns:
            True if signature is valid, False otherwise
        """
        try:
            if isinstance(payload, dict):
                payload_string = json.dumps(payload, sort_keys=True)
            else:
                payload_string = str(payload)

            expected_signature = hmac.new(
                secret.encode("utf-8"),
                payload_string.encode("utf-8"),
                hashlib.sha256,
            ).hexdigest()

            return hmac.compare_digest(expected_signature, signature)
        except Exception:
            return False




"""Payment Links resource."""

from typing import Optional, Dict, Any
from ..http_client import HttpClient


class PaymentLinks:
    """Payment Links resource."""

    def __init__(self, http: HttpClient):
        self.http = http

    def list(
        self,
        page: Optional[int] = None,
        limit: Optional[int] = None,
        status: Optional[str] = None,
    ) -> list:
        """List all payment links.
        
        Args:
            page: Page number for pagination
            limit: Number of items per page
            status: Filter by status (active, inactive, archived)
            
        Returns:
            List of payment link dictionaries
        """
        params = {}
        if page is not None:
            params["page"] = page
        if limit is not None:
            params["limit"] = limit
        if status:
            params["status"] = status

        return self.http.get("/payment-links", params=params)

    def retrieve(self, link_id: str) -> Dict[str, Any]:
        """Retrieve a specific payment link.
        
        Args:
            link_id: The payment link ID to retrieve
            
        Returns:
            Payment link dictionary
        """
        return self.http.get(f"/payment-links/{link_id}")

    def create(
        self,
        title: str,
        amount: float,
        description: Optional[str] = None,
        currency: str = "KES",
        allow_custom_amount: bool = False,
        min_amount: Optional[float] = None,
        max_amount: Optional[float] = None,
        success_redirect_url: Optional[str] = None,
        cancel_redirect_url: Optional[str] = None,
        expires_at: Optional[str] = None,
    ) -> Dict[str, Any]:
        """Create a new payment link.
        
        Args:
            title: Payment link title
            amount: Payment amount
            description: Optional description
            currency: Currency code (default: KES)
            allow_custom_amount: Allow customers to enter custom amount
            min_amount: Minimum amount if allow_custom_amount is True
            max_amount: Maximum amount if allow_custom_amount is True
            success_redirect_url: URL to redirect after successful payment
            cancel_redirect_url: URL to redirect if payment is cancelled
            expires_at: Expiration date (ISO 8601 format)
            
        Returns:
            Created payment link dictionary
        """
        data = {
            "title": title,
            "amount": amount,
            "currency": currency,
            "allowCustomAmount": allow_custom_amount,
        }
        if description:
            data["description"] = description
        if min_amount is not None:
            data["minAmount"] = min_amount
        if max_amount is not None:
            data["maxAmount"] = max_amount
        if success_redirect_url:
            data["successRedirectUrl"] = success_redirect_url
        if cancel_redirect_url:
            data["cancelRedirectUrl"] = cancel_redirect_url
        if expires_at:
            data["expiresAt"] = expires_at

        return self.http.post("/payment-links", data=data)

    def update(
        self,
        link_id: str,
        title: Optional[str] = None,
        description: Optional[str] = None,
        amount: Optional[float] = None,
        status: Optional[str] = None,
        allow_custom_amount: Optional[bool] = None,
        min_amount: Optional[float] = None,
        max_amount: Optional[float] = None,
        success_redirect_url: Optional[str] = None,
        cancel_redirect_url: Optional[str] = None,
        expires_at: Optional[str] = None,
    ) -> Dict[str, Any]:
        """Update a payment link.
        
        Args:
            link_id: The payment link ID to update
            title: Updated title
            description: Updated description
            amount: Updated amount
            status: Updated status (active, inactive, archived)
            allow_custom_amount: Updated allow custom amount flag
            min_amount: Updated minimum amount
            max_amount: Updated maximum amount
            success_redirect_url: Updated success redirect URL
            cancel_redirect_url: Updated cancel redirect URL
            expires_at: Updated expiration date
            
        Returns:
            Updated payment link dictionary
        """
        data = {}
        if title is not None:
            data["title"] = title
        if description is not None:
            data["description"] = description
        if amount is not None:
            data["amount"] = amount
        if status is not None:
            data["status"] = status
        if allow_custom_amount is not None:
            data["allowCustomAmount"] = allow_custom_amount
        if min_amount is not None:
            data["minAmount"] = min_amount
        if max_amount is not None:
            data["maxAmount"] = max_amount
        if success_redirect_url is not None:
            data["successRedirectUrl"] = success_redirect_url
        if cancel_redirect_url is not None:
            data["cancelRedirectUrl"] = cancel_redirect_url
        if expires_at is not None:
            data["expiresAt"] = expires_at

        return self.http.put(f"/payment-links/{link_id}", data=data)

    def delete(self, link_id: str) -> None:
        """Delete a payment link.
        
        Args:
            link_id: The payment link ID to delete
        """
        self.http.delete(f"/payment-links/{link_id}")

    def get_public(self, slug: str) -> Dict[str, Any]:
        """Get public payment link (no authentication required).
        
        Args:
            slug: The payment link slug
            
        Returns:
            Payment link dictionary
        """
        return self.http.get(f"/payment-links/public/{slug}")

    def initiate_payment(
        self,
        slug: str,
        phone: str,
        amount: Optional[float] = None,
    ) -> Dict[str, Any]:
        """Initiate payment on a payment link.
        
        Args:
            slug: The payment link slug
            phone: Customer phone number
            amount: Optional custom amount (if allow_custom_amount is True)
            
        Returns:
            Payment initiation response
        """
        data = {"phone": phone}
        if amount is not None:
            data["amount"] = amount

        return self.http.post(f"/payment-links/public/{slug}/pay", data=data)




"""Main Lipana client."""

from typing import Literal, Optional
from .http_client import HttpClient
from .resources.transactions import Transactions
from .resources.payment_links import PaymentLinks
from .resources.webhooks import Webhooks
from .resources.api_keys import ApiKeys


class Lipana:
    """Main Lipana SDK client."""

    VERSION = "1.0.1"

    def __init__(
        self,
        api_key: str,
        environment: Literal["sandbox", "production"] = "production",
        base_url: Optional[str] = None,
    ):
        """Initialize the Lipana SDK client.
        
        Args:
            api_key: Your Lipana API secret key
            environment: Environment to use (sandbox or production)
            base_url: Optional custom base URL (overrides default)
            
        Raises:
            ValueError: If api_key is empty
        """
        if not api_key:
            raise ValueError("Lipana API key is required")

        self.api_key = api_key
        self.environment = environment
        self.base_url = base_url or self._get_default_base_url()

        # Initialize HTTP client
        http = HttpClient(self.api_key, self.base_url, self.environment)

        # Initialize resources
        self.transactions = Transactions(http)
        self.payment_links = PaymentLinks(http)
        self.webhooks = Webhooks(http)
        self.api_keys = ApiKeys(http)

    def _get_default_base_url(self) -> str:
        """Get default base URL based on environment."""
        if self.environment == "sandbox":
            return "https://api-sandbox.lipana.dev/v1"
        return "https://api.lipana.dev/v1"




"""Tests for Lipana client."""

import pytest
from lipana import Lipana, LipanaError


def test_client_initialization():
    """Test client initialization."""
    client = Lipana(api_key="test-key", environment="sandbox")
    assert client.api_key == "test-key"
    assert client.environment == "sandbox"
    assert client.base_url == "https://api-sandbox.lipana.dev/api"


def test_client_production_environment():
    """Test client with production environment."""
    client = Lipana(api_key="test-key", environment="production")
    assert client.base_url == "https://api.lipana.dev/api"


def test_client_custom_base_url():
    """Test client with custom base URL."""
    client = Lipana(
        api_key="test-key",
        base_url="https://custom-api.example.com/api"
    )
    assert client.base_url == "https://custom-api.example.com/api"


def test_missing_api_key():
    """Test that missing API key raises error."""
    with pytest.raises(ValueError, match="API key is required"):
        Lipana(api_key="")


def test_resources_initialized():
    """Test that all resources are initialized."""
    client = Lipana(api_key="test-key")
    assert client.transactions is not None
    assert client.payment_links is not None
    assert client.webhooks is not None
    assert client.api_keys is not None


def test_version():
    """Test SDK version."""
    assert Lipana.VERSION == "1.0.0"




"""Custom exceptions for the Lipana SDK."""


class LipanaError(Exception):
    """Base exception for all Lipana SDK errors."""

    def __init__(self, message: str, status_code: int = None, code: str = None, errors: dict = None):
        super().__init__(message)
        self.message = message
        self.status_code = status_code
        self.code = code
        self.errors = errors or {}

    def is_authentication_error(self) -> bool:
        """Check if error is due to authentication failure."""
        return self.status_code == 401 or self.code == "AUTHENTICATION_FAILED"

    def is_validation_error(self) -> bool:
        """Check if error is due to validation failure."""
        return self.status_code == 400 or self.code == "VALIDATION_ERROR"

    def is_rate_limit_error(self) -> bool:
        """Check if error is due to rate limiting."""
        return self.status_code == 429 or self.code == "RATE_LIMIT_EXCEEDED"

    def is_server_error(self) -> bool:
        """Check if error is a server error."""
        return self.status_code is not None and self.status_code >= 500




"""HTTP client for making API requests."""

import requests
from typing import Any, Dict, Optional
from .errors import LipanaError


class HttpClient:
    """HTTP client for Lipana API requests."""

    def __init__(self, api_key: str, base_url: str, environment: str):
        self.api_key = api_key
        self.base_url = base_url
        self.environment = environment
        self.session = requests.Session()
        self.session.headers.update({
            "Content-Type": "application/json",
            "User-Agent": "Lipana-Python-SDK/1.0.0",
            "x-api-key": api_key,
        })

    def _handle_response(self, response: requests.Response) -> Any:
        """Handle API response and raise errors if needed."""
        try:
            data = response.json()
        except ValueError:
            data = {}

        if not response.ok:
            message = data.get("message", response.reason or "An error occurred")
            code = data.get("code")
            errors = data.get("errors", {})
            raise LipanaError(message, response.status_code, code, errors)

        return data.get("data", data)

    def get(self, path: str, params: Optional[Dict] = None) -> Any:
        """Make a GET request."""
        url = f"{self.base_url}{path}"
        response = self.session.get(url, params=params, timeout=30)
        return self._handle_response(response)

    def post(self, path: str, data: Optional[Dict] = None) -> Any:
        """Make a POST request."""
        url = f"{self.base_url}{path}"
        response = self.session.post(url, json=data, timeout=30)
        return self._handle_response(response)

    def put(self, path: str, data: Optional[Dict] = None) -> Any:
        """Make a PUT request."""
        url = f"{self.base_url}{path}"
        response = self.session.put(url, json=data, timeout=30)
        return self._handle_response(response)

    def delete(self, path: str) -> Any:
        """Make a DELETE request."""
        url = f"{self.base_url}{path}"
        response = self.session.delete(url, timeout=30)
        return self._handle_response(response)




"""Setup configuration for Lipana Python SDK."""

from setuptools import setup, find_packages

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setup(
    name="lipana",
    version="1.0.1",
    author="Lipana",
    author_email="dev@lipana.dev",
    description="Official Python SDK for Lipana M-Pesa API",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/lipana/lipana-python",
    packages=find_packages(),
    classifiers=[
        "Development Status :: 4 - Beta",
        "Intended Audience :: Developers",
        "Topic :: Software Development :: Libraries :: Python Modules",
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
        "Programming Language :: Python :: 3.12",
    ],
    python_requires=">=3.8",
    install_requires=[
        "requests>=2.28.0",
    ],
    extras_require={
        "dev": [
            "pytest>=7.0.0",
            "pytest-cov>=4.0.0",
            "black>=22.0.0",
            "mypy>=0.991",
        ],
    },
    keywords=["lipana", "mpesa", "payments", "kenya", "stk-push", "api", "sdk"],
    project_urls={
        "Homepage": "https://lipana.dev",
        "Documentation": "https://lipana.dev/docs",
        "Repository": "https://github.com/lipana/lipana-python",
        "Issues": "https://github.com/lipana/lipana-python/issues",
    },
)




"""Transactions resource."""

from typing import Optional, Dict, Any
from ..http_client import HttpClient


class Transactions:
    """Transactions resource."""

    def __init__(self, http: HttpClient):
        self.http = http

    def list(
        self,
        page: Optional[int] = None,
        limit: Optional[int] = None,
        status: Optional[str] = None,
        start_date: Optional[str] = None,
        end_date: Optional[str] = None,
    ) -> Dict[str, Any]:
        """List all transactions.
        
        Args:
            page: Page number for pagination
            limit: Number of items per page
            status: Filter by status (pending, success, failed, cancelled)
            start_date: Filter transactions from this date (YYYY-MM-DD)
            end_date: Filter transactions until this date (YYYY-MM-DD)
            
        Returns:
            Dictionary containing transactions and pagination info
        """
        params = {}
        if page is not None:
            params["page"] = page
        if limit is not None:
            params["limit"] = limit
        if status:
            params["status"] = status
        if start_date:
            params["startDate"] = start_date
        if end_date:
            params["endDate"] = end_date

        return self.http.get("/transactions", params=params)

    def retrieve(self, transaction_id: str) -> Dict[str, Any]:
        """Retrieve a specific transaction.
        
        Args:
            transaction_id: The transaction ID to retrieve
            
        Returns:
            Transaction dictionary
        """
        return self.http.get(f"/transactions/{transaction_id}")

    def create(
        self,
        amount: float,
        phone: str,
        account_reference: Optional[str] = None,
        transaction_desc: Optional[str] = None,
    ) -> Dict[str, Any]:
        """Create a new transaction.
        
        Args:
            amount: Transaction amount
            phone: Customer phone number (format: +254xxxxxxxxx)
            account_reference: Optional account reference
            transaction_desc: Optional transaction description
            
        Returns:
            Created transaction dictionary
        """
        data = {
            "amount": amount,
            "phone": phone,
        }
        if account_reference:
            data["accountReference"] = account_reference
        if transaction_desc:
            data["transactionDesc"] = transaction_desc

        return self.http.post("/transactions", data=data)

    def initiate_stk_push(
        self,
        phone: str,
        amount: float,
        account_reference: Optional[str] = None,
        transaction_desc: Optional[str] = None,
    ) -> Dict[str, Any]:
        """Initiate an STK push payment.
        
        Args:
            phone: Customer phone number (format: +254xxxxxxxxx)
            amount: Payment amount
            account_reference: Optional account reference
            transaction_desc: Optional transaction description
            
        Returns:
            STK push response dictionary
        """
        data = {
            "phone": phone,
            "amount": amount,
        }
        if account_reference:
            data["accountReference"] = account_reference
        if transaction_desc:
            data["transactionDesc"] = transaction_desc

        return self.http.post("/transactions/push-stk", data=data)

    def update_status(
        self,
        transaction_id: str,
        status: str,
    ) -> Dict[str, Any]:
        """Update transaction status.
        
        Args:
            transaction_id: The transaction ID to update
            status: New status (pending, success, failed, cancelled)
            
        Returns:
            Updated transaction dictionary
        """
        return self.http.put(f"/transactions/{transaction_id}/status", data={"status": status})

